import pandas as pd
import json
from sklearn.feature_selection import VarianceThreshold

symbols = {
    'ADA': 'source/ada.csv',
    'BCH': 'source/bch.csv',
    'BNB': 'source/bnb.csv',
    # 'BNB-MAIN': 'source/bnb_mainnet.csv',  # Starts from 2019-04-18 so no impact here
    'BTC': 'source/btc.csv',
    'BTG': 'source/btg.csv',
    'DASH': 'source/dash.csv',
    'DOGE': 'source/doge.csv',
    'EOS': 'source/eos_eth.csv',
    #'EOS-MAIN': 'source/eos.csv',
    'ETC': 'source/etc.csv',
    'ETH': 'source/eth.csv',
    'LINK': 'source/link.csv',
    'LTC': 'source/ltc.csv',
    'NEO': 'source/neo.csv',
    'QTUM': 'source/qtum.csv',
    'TRX': 'source/trx.csv', # 2018-06-25 onwards
    'TRX-ETH': 'source/trx_eth.csv', # Upto 2018-06-25
    # 'USDT': 'source/usdt.csv',
    # 'USDT-ETH': 'source/usdt_eth.csv',
    'VEN': 'source/ven.csv',
    'WAVES': 'source/waves.csv',
    'XEM': 'source/xem.csv',
    'XMR': 'source/xmr.csv',
    'XRP': 'source/xrp.csv',
    'ZEC': 'source/zec.csv',
    'ZRX': 'source/zrx.csv',
}

if __name__ == '__main__':
    index = {}
    for sym, csv in symbols.items():

        df = pd.read_csv(csv, sep=',', encoding='utf-8', index_col='date', parse_dates=True)
        df.columns = [c.lower().replace('.','_') for c in df.columns]
        # Drop columns containing nans, keep those with at least 30 non-na values
        # Drop rows containing nans, keep those with at least 5 non-na values
        df = df.dropna(thresh=30, axis='columns').dropna(thresh=5)
        # Remove zero-variance features
        sel = VarianceThreshold()
        sel.fit_transform(df)
        sup = sel.get_support()
        df = df[[name for flag, name in zip(sup, df.columns) if flag]]

        csv_path = 'csv/{}.csv'.format(sym.lower())
        xls_path = 'excel/{}.xlsx'.format(sym.lower())
        df.to_csv(csv_path, sep=',', encoding='utf-8', index=True, index_label='Date')
        df.to_excel(xls_path, index=True, index_label='Date')
        index[sym] = {'csv':csv_path, 'xls':xls_path}
        print('Saved {}'.format(sym))

    with open('index.json', 'w') as f:
        json.dump(index, f, sort_keys=True, indent=4)
